@@ -11,6 +11,7 @@
 
 #include <type_traits>
 
+#include "../types/xsimd_base.hpp"
 #include "xsimd_types_include.hpp"
 
 #undef XSIMD_BATCH_INT_SIZE
@@ -19,7 +20,7 @@
 
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_AVX512_VERSION
 #define XSIMD_BATCH_INT8_SIZE 64
-// #define XSIMD_BATCH_INT16_SIZE 32
+#define XSIMD_BATCH_INT16_SIZE 32
 #define XSIMD_BATCH_INT32_SIZE 16
 #define XSIMD_BATCH_INT64_SIZE 8
 #define XSIMD_BATCH_FLOAT_SIZE 16
@@ -40,7 +41,7 @@
 #define XSIMD_BATCH_DOUBLE_SIZE 2
 #elif XSIMD_ARM_INSTR_SET >= XSIMD_ARM7_NEON_VERSION
 #define XSIMD_BATCH_INT8_SIZE 16
-// #define XSIMD_BATCH_INT16_SIZE 8
+#define XSIMD_BATCH_INT16_SIZE 8
 #define XSIMD_BATCH_INT32_SIZE 4
 #define XSIMD_BATCH_INT64_SIZE 2
 #define XSIMD_BATCH_FLOAT_SIZE 4
@@ -52,7 +53,6 @@
 
 namespace xsimd
 {
-
     template <class T>
     struct simd_traits
     {
@@ -108,6 +108,36 @@ namespace xsimd
         static constexpr size_t size = simd_traits<type>::size;
     };
 
+    template <>
+    struct simd_traits<int16_t>
+    {
+        using type = batch<int16_t, XSIMD_BATCH_INT16_SIZE>;
+        using bool_type = simd_batch_traits<type>::batch_bool_type;
+        static constexpr size_t size = type::size;
+    };
+
+    template <>
+    struct revert_simd_traits<batch<int16_t, XSIMD_BATCH_INT16_SIZE>>
+    {
+        using type = int16_t;
+        static constexpr size_t size = simd_traits<type>::size;
+    };
+
+    template <>
+    struct simd_traits<uint16_t>
+    {
+        using type = batch<uint16_t, XSIMD_BATCH_INT16_SIZE>;
+        using bool_type = simd_batch_traits<type>::batch_bool_type;
+        static constexpr size_t size = type::size;
+    };
+
+    template <>
+    struct revert_simd_traits<batch<uint16_t, XSIMD_BATCH_INT16_SIZE>>
+    {
+        using type = uint16_t;
+        static constexpr size_t size = simd_traits<type>::size;
+    };
+
     template <>
     struct simd_traits<int32_t>
     {
@@ -123,6 +153,48 @@ namespace xsimd
         static constexpr size_t size = simd_traits<type>::size;
     };
 
+    template <>
+    struct simd_traits<uint32_t>
+    {
+        using type = batch<uint32_t, XSIMD_BATCH_INT32_SIZE>;
+        using bool_type = simd_batch_traits<type>::batch_bool_type;
+        static constexpr size_t size = type::size;
+    };
+
+    template <>
+    struct revert_simd_traits<batch<uint32_t, XSIMD_BATCH_INT32_SIZE>>
+    {
+        using type = uint32_t;
+        static constexpr size_t size = simd_traits<type>::size;
+    };
+
+// On some architectures long is a different type from int32_t or int64_t
+#ifdef XSIMD_32_BIT_ABI
+    template <>
+    struct simd_traits<long> : simd_traits<int32_t>
+    {
+    };
+
+    template <>
+    struct revert_simd_traits<batch<long, XSIMD_BATCH_INT32_SIZE>>
+    {
+        using type = long;
+        static constexpr size_t size = simd_traits<type>::size;
+    };
+
+    template <>
+    struct simd_traits<unsigned long> : simd_traits<uint32_t>
+    {
+    };
+
+    template <>
+    struct revert_simd_traits<batch<unsigned long, XSIMD_BATCH_INT32_SIZE>>
+    {
+        using type = unsigned long;
+        static constexpr size_t size = simd_traits<type>::size;
+    };
+#endif // XSIMD_32_BIT_ABI
+
     template <>
     struct simd_traits<int64_t>
     {
@@ -138,6 +210,21 @@ namespace xsimd
         static constexpr size_t size = simd_traits<type>::size;
     };
 
+    template <>
+    struct simd_traits<uint64_t>
+    {
+        using type = batch<uint64_t, XSIMD_BATCH_INT64_SIZE>;
+        using bool_type = simd_batch_traits<type>::batch_bool_type;
+        static constexpr size_t size = type::size;
+    };
+
+    template <>
+    struct revert_simd_traits<batch<uint64_t, XSIMD_BATCH_INT64_SIZE>>
+    {
+        using type = uint64_t;
+        static constexpr size_t size = simd_traits<type>::size;
+    };
+
     template <>
     struct simd_traits<float>
     {
@@ -183,71 +270,8 @@ namespace xsimd
         using type = xtl::xcomplex<float, float, i3ec>;
         static constexpr size_t size = simd_traits<type>::size;
     };
-#endif
-#endif
-
-// TODO remove when we've implemented these types for AVX512 / ARM NEON!
-#ifdef XSIMD_BATCH_INT16_SIZE
-    template <>
-    struct simd_traits<int16_t>
-    {
-        using type = batch<int16_t, XSIMD_BATCH_INT16_SIZE>;
-        using bool_type = simd_batch_traits<type>::batch_bool_type;
-        static constexpr size_t size = type::size;
-    };
-
-    template <>
-    struct revert_simd_traits<batch<int16_t, XSIMD_BATCH_INT16_SIZE>>
-    {
-        using type = int16_t;
-        static constexpr size_t size = simd_traits<type>::size;
-    };
-
-    template <>
-    struct simd_traits<uint16_t>
-    {
-        using type = batch<uint16_t, XSIMD_BATCH_INT16_SIZE>;
-        using bool_type = simd_batch_traits<type>::batch_bool_type;
-        static constexpr size_t size = type::size;
-    };
-
-    template <>
-    struct revert_simd_traits<batch<uint16_t, XSIMD_BATCH_INT16_SIZE>>
-    {
-        using type = uint16_t;
-        static constexpr size_t size = simd_traits<type>::size;
-    };
-
-    template <>
-    struct simd_traits<uint32_t>
-    {
-        using type = batch<uint32_t, XSIMD_BATCH_INT32_SIZE>;
-        using bool_type = simd_batch_traits<type>::batch_bool_type;
-        static constexpr size_t size = type::size;
-    };
-
-    template <>
-    struct revert_simd_traits<batch<uint32_t, XSIMD_BATCH_INT32_SIZE>>
-    {
-        using type = uint32_t;
-        static constexpr size_t size = simd_traits<type>::size;
-    };
-
-    template <>
-    struct simd_traits<uint64_t>
-    {
-        using type = batch<uint64_t, XSIMD_BATCH_INT64_SIZE>;
-        using bool_type = simd_batch_traits<type>::batch_bool_type;
-        static constexpr size_t size = type::size;
-    };
-
-    template <>
-    struct revert_simd_traits<batch<uint64_t, XSIMD_BATCH_INT64_SIZE>>
-    {
-        using type = uint64_t;
-        static constexpr size_t size = simd_traits<type>::size;
-    };
-#endif
+#endif // XSIMD_ENABLE_XTL_COMPLEX
+#endif // XSIMD_BATCH_FLOAT_SIZE
 
 #ifdef XSIMD_BATCH_DOUBLE_SIZE
     template <>
@@ -295,8 +319,8 @@ namespace xsimd
         using type = xtl::xcomplex<double, double, i3ec>;
         static constexpr size_t size = simd_traits<type>::size;
     };
-#endif
-#endif
+#endif // XSIMD_ENABLE_XTL_COMPLEX
+#endif // XSIMD_BATCH_DOUBLE_SIZE
 
     /********************
      * simd_return_type *
@@ -316,8 +340,10 @@ namespace xsimd
                 std::is_same<T1, uint8_t>::value ||
                 std::is_same<T1, int16_t>::value ||
                 std::is_same<T1, uint16_t>::value ||
-                std::is_same<T1, int64_t>::value ||
                 std::is_same<T1, int32_t>::value ||
+                std::is_same<T1, uint32_t>::value ||
+                std::is_same<T1, int64_t>::value ||
+                std::is_same<T1, uint64_t>::value ||
                 std::is_same<T1, char>::value ||
                 detail::is_complex<T1>::value;
         };
@@ -358,7 +384,7 @@ namespace xsimd
             : std::enable_if<simd_condition<T1, T2>::value, batch<xtl::xcomplex<T2, T2, i3ec>, N>>
         {
         };
-#endif
+#endif // XSIMD_ENABLE_XTL_COMPLEX
     }
 
     template <class T1, class T2, std::size_t N = simd_traits<T2>::size>
@@ -397,7 +423,7 @@ namespace xsimd
     struct is_batch_complex<batch<xtl::xcomplex<T, T, i3ec>, N>> : std::true_type
     {
     };
-#endif
+#endif //XSIMD_ENABLE_XTL_COMPLEX
 
 }
 
